import { UrlCollection, } from "./url-collection.js";
// interface definitions
// ---------------------
var FetchpHookType;
(function (FetchpHookType) {
    FetchpHookType["BuildRequestHeaders"] = "BuildRequestHeaders";
    FetchpHookType["NewRequest"] = "NewRequest";
    FetchpHookType["StateChange"] = "StateChange";
    FetchpHookType["Success"] = "Success";
    FetchpHookType["Error"] = "Error";
    FetchpHookType["Cancel"] = "Cancel";
})(FetchpHookType || (FetchpHookType = {}));
class HookRegistry {
    constructor() {
        Object.defineProperty(this, "items", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.items = {};
    }
    internalEnsureHookItemExists(hookType) {
        if (!(hookType in this.items)) {
            this.items[hookType] = {
                globals: new Set(),
                urlBased: new UrlCollection(),
            };
        }
    }
    add(hookType, func) {
        this.internalEnsureHookItemExists(hookType);
        this.items[hookType].globals.add(func);
    }
    addForUrl(hookType, methods, requestUrlPattern, func) {
        this.internalEnsureHookItemExists(hookType);
        this.items[hookType].urlBased.add(methods, requestUrlPattern, func);
    }
    clear() {
        this.items = {};
    }
    // deno-lint-ignore no-explicit-any
    async callGlobalHooks(hookType, ...params) {
        if (!(hookType in this.items)) {
            return;
        }
        for (const func of this.items[hookType].globals) {
            await func(...params);
        }
        return undefined;
    }
    async callHooksWithRequest(hookType, request, urlConverter, 
    // deno-lint-ignore no-explicit-any
    ...params) {
        if (!(hookType in this.items)) {
            return;
        }
        for (const func of this.items[hookType].globals) {
            await func(...params);
        }
        if (request !== undefined) {
            for (const rule of this.items[hookType].urlBased.filterByRequest(request, urlConverter)) {
                await rule.data(...params);
            }
        }
        return undefined;
    }
}
export { FetchpHookType, HookRegistry, };
