import { React } from "./deps.js";
import { fetchp, FetchpStatus, } from "./fetchp.js";
const useFetchpBuilder = function useFetchpBuilder(fetchpInstance) {
    // deno-lint-ignore no-explicit-any
    return function useFetchp(method, url, init) {
        var _a;
        const [data, setData] = React.useState();
        const [status, setStatus] = React.useState(FetchpStatus.IDLE);
        // deno-lint-ignore no-explicit-any
        const [error, setError] = React.useState();
        const [result, setResult] = React.useState();
        const [doFetch, setDoFetch] = React.useState((_a = init === null || init === void 0 ? void 0 : init.autoFetch) !== null && _a !== void 0 ? _a : true);
        React.useEffect(() => {
            if (!doFetch) {
                return;
            }
            const fetchData = async function fetchData() {
                // try {
                //   const result = fetchpInstance.request<T>(method, url, {
                //     ...(init ?? {}),
                //     autoFetch: true,
                //     statusCallback: (newStatus) => {
                //       setStatus(newStatus);
                //     },
                //     errorCallback: (newError) => setError(newError),
                //     successCallback: (newData) => setData(newData),
                //     // cancelCallback: () => {},
                //   });
                //   setResult(result);
                // } catch (error) {
                //   setError(error);
                //   console.error(error);
                //   setStatus(FetchpStatus.ERROR);
                // }
                setError(undefined);
                setStatus(FetchpStatus.FETCHING);
                try {
                    const result = fetchpInstance.request(method, url, Object.assign(Object.assign({}, (init !== null && init !== void 0 ? init : {})), { autoFetch: true }));
                    setResult(result);
                    setData(await result.data);
                    if (result.abortController.signal.aborted) {
                        setStatus(FetchpStatus.CANCELED);
                    }
                    else {
                        setStatus(FetchpStatus.SUCCESS);
                    }
                }
                catch (error) {
                    setError(error);
                    console.error(error);
                    setStatus(FetchpStatus.ERROR);
                }
            };
            fetchData();
        }, [doFetch]);
        return {
            doFetch: () => setDoFetch(true),
            data,
            status,
            isIdle: (status === FetchpStatus.IDLE),
            isFetching: (status === FetchpStatus.FETCHING),
            isLoading: (status === FetchpStatus.LOADING),
            isError: (status === FetchpStatus.ERROR),
            isCanceled: (status === FetchpStatus.CANCELED),
            isSuccess: (status === FetchpStatus.SUCCESS),
            error,
            result,
        };
    };
};
const useFetchp = useFetchpBuilder(fetchp);
export { FetchpStatus, useFetchp, useFetchp as default, useFetchpBuilder };
