"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFetchpBuilder = exports.default = exports.useFetchp = exports.FetchpStatus = void 0;
const deps_js_1 = require("./deps.js");
const fetchp_js_1 = require("./fetchp.js");
Object.defineProperty(exports, "FetchpStatus", { enumerable: true, get: function () { return fetchp_js_1.FetchpStatus; } });
const useFetchpBuilder = function useFetchpBuilder(fetchpInstance) {
    // deno-lint-ignore no-explicit-any
    return function useFetchp(method, url, init) {
        var _a;
        const [data, setData] = deps_js_1.React.useState();
        const [status, setStatus] = deps_js_1.React.useState(fetchp_js_1.FetchpStatus.IDLE);
        // deno-lint-ignore no-explicit-any
        const [error, setError] = deps_js_1.React.useState();
        const [result, setResult] = deps_js_1.React.useState();
        const [doFetch, setDoFetch] = deps_js_1.React.useState((_a = init === null || init === void 0 ? void 0 : init.autoFetch) !== null && _a !== void 0 ? _a : true);
        deps_js_1.React.useEffect(() => {
            if (!doFetch) {
                return;
            }
            const fetchData = async function fetchData() {
                // try {
                //   const result = fetchpInstance.request<T>(method, url, {
                //     ...(init ?? {}),
                //     autoFetch: true,
                //     statusCallback: (newStatus) => {
                //       setStatus(newStatus);
                //     },
                //     errorCallback: (newError) => setError(newError),
                //     successCallback: (newData) => setData(newData),
                //     // cancelCallback: () => {},
                //   });
                //   setResult(result);
                // } catch (error) {
                //   setError(error);
                //   console.error(error);
                //   setStatus(FetchpStatus.ERROR);
                // }
                setError(undefined);
                setStatus(fetchp_js_1.FetchpStatus.FETCHING);
                try {
                    const result = fetchpInstance.request(method, url, Object.assign(Object.assign({}, (init !== null && init !== void 0 ? init : {})), { autoFetch: true }));
                    setResult(result);
                    setData(await result.data);
                    if (result.abortController.signal.aborted) {
                        setStatus(fetchp_js_1.FetchpStatus.CANCELED);
                    }
                    else {
                        setStatus(fetchp_js_1.FetchpStatus.SUCCESS);
                    }
                }
                catch (error) {
                    setError(error);
                    console.error(error);
                    setStatus(fetchp_js_1.FetchpStatus.ERROR);
                }
            };
            fetchData();
        }, [doFetch]);
        return {
            doFetch: () => setDoFetch(true),
            data,
            status,
            isIdle: (status === fetchp_js_1.FetchpStatus.IDLE),
            isFetching: (status === fetchp_js_1.FetchpStatus.FETCHING),
            isLoading: (status === fetchp_js_1.FetchpStatus.LOADING),
            isError: (status === fetchp_js_1.FetchpStatus.ERROR),
            isCanceled: (status === fetchp_js_1.FetchpStatus.CANCELED),
            isSuccess: (status === fetchp_js_1.FetchpStatus.SUCCESS),
            error,
            result,
        };
    };
};
exports.useFetchpBuilder = useFetchpBuilder;
const useFetchp = useFetchpBuilder(fetchp_js_1.fetchp);
exports.useFetchp = useFetchp;
exports.default = useFetchp;
