import * as dntShim from "./_dnt.shims.js";
import { type FetchpHookFn, FetchpHookType, HookRegistry, type HookRegistryInterface } from "./hook-registry.js";
import { MockRegistry, type MockRegistryInterface } from "./mock-registry.js";
import { CacheRegistry, type CacheRegistryInterface } from "./cache-registry.js";
declare enum FetchpStatus {
    IDLE = "idle",
    PREPARING = "preparing",
    FETCHING = "fetching",
    LOADING = "loading",
    ERROR = "error",
    CANCELED = "canceled",
    SUCCESS = "success"
}
interface FetchpRequestInit extends dntShim.RequestInit {
    autoFetch?: boolean;
    cacheRequest?: boolean;
    statusCallback?: (status: FetchpStatus) => void;
    successCallback?: (data: any) => void;
    errorCallback?: (error: any) => void;
    cancelCallback?: () => void;
}
interface FetchpResultInterface<T = any> {
    readonly request: dntShim.Request | undefined;
    readonly response: Promise<dntShim.Response | undefined>;
    abortController: AbortController;
    readonly status: FetchpStatus;
    readonly error: any;
    readonly data: Promise<T | undefined>;
    exec(): FetchpResultInterface<T>;
}
interface FetchpInterface {
    baseUrl: string | undefined;
    hooks: HookRegistryInterface;
    mocks: MockRegistryInterface;
    cache: CacheRegistryInterface;
    setBaseUrl: (url: string) => void;
    request: <T = any>(method: string, url: string, init?: FetchpRequestInit) => FetchpResultInterface<T>;
}
declare type InternalFetchState = [
    status: FetchpStatus,
    request: dntShim.Request | undefined,
    response: dntShim.Response | undefined,
    error: any,
    ...others: any[]
];
declare class Fetchp implements FetchpInterface {
    baseUrl: string | undefined;
    hooks: HookRegistryInterface;
    mocks: MockRegistryInterface;
    cache: CacheRegistryInterface;
    constructor();
    setBaseUrl(url: string | undefined): void;
    internalMockChecker(request: dntShim.Request): Promise<dntShim.Response> | undefined;
    internalCacheChecker(request: dntShim.Request): Promise<dntShim.Response> | undefined;
    internalFetcher(request: dntShim.Request): Promise<dntShim.Response>;
    internalDataDeserializer<T>(response: dntShim.Response, cacheMode: boolean): Promise<T>;
    internalUrlConverter(url: string): URL;
    internalAwaiterGenerator<T = unknown>(): [(value: T) => void, Promise<T>];
    internalRequestStep1InitHeaders(headers: dntShim.Headers, init: FetchpRequestInit | undefined, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep2InitRequest(method: string, url: URL, headers: dntShim.Headers, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus, request: dntShim.Request) => void): Promise<InternalFetchState>;
    internalRequestStep3CheckForMocks(request: dntShim.Request | undefined, abortController: AbortController, _callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep4CheckForCache(request: dntShim.Request | undefined, mockedResponse: dntShim.Response | undefined, abortController: AbortController, _callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestStep5ExecuteRequest(request: dntShim.Request | undefined, mockedOrCachedResponse: dntShim.Response | undefined, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalRequestOnError(request: dntShim.Request | undefined, error: any, callback: (status: FetchpStatus, error: any) => void): Promise<InternalFetchState>;
    internalLoadStep1CheckState(status: FetchpStatus, request: dntShim.Request | undefined, response: dntShim.Response | undefined, error: any, init: FetchpRequestInit | undefined, abortController: AbortController, callback: (status: FetchpStatus) => void): Promise<InternalFetchState>;
    internalLoadStep2Deserialization<T = any>(status: FetchpStatus, request: dntShim.Request | undefined, response: dntShim.Response | undefined, error: any, init: FetchpRequestInit | undefined, callback: (status: FetchpStatus, error: any) => void): Promise<[FetchpStatus, T | undefined, any, ...any[]]>;
    request<T = any>(method: string, url: string, init?: FetchpRequestInit): {
        readonly request: dntShim.Request | undefined;
        readonly response: Promise<dntShim.Response | undefined>;
        abortController: AbortController;
        readonly status: FetchpStatus;
        readonly error: any;
        readonly data: Promise<T | undefined>;
        exec: () => any;
    };
}
declare const fetchp: Fetchp;
export { CacheRegistry, type CacheRegistryInterface, Fetchp, fetchp, fetchp as default, type FetchpHookFn, FetchpHookType, type FetchpInterface, type FetchpRequestInit, type FetchpResultInterface, FetchpStatus, HookRegistry, type HookRegistryInterface, MockRegistry, type MockRegistryInterface, };
